@@ -491,7 +491,6 @@ char   *vstring_memchr(VSTRING *vp, int ch)
 
 VSTRING *vstring_insert(VSTRING *vp, ssize_t start, const char *buf, ssize_t len)
 {
-    const char *myname = "vstring_insert";
     ssize_t new_len;
 
     /*
@@ -625,6 +624,7 @@ VSTRING *vstring_sprintf_prepend(VSTRING *vp, const char *format,...)
     result_len = VSTRING_LEN(vp);
 
     /* Construct: old|new|old|free */
+    VSTRING_SPACE(vp, old_len);
     vstring_memcat(vp, vstring_str(vp), old_len);
 
     /* Construct: new|old|free */
